# Generated by Django 5.0.7 on 2024-08-14 19:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0055_alter_agent_style_icon"),
    ]

    operations = [
        migrations.AddField(
            model_name="chatmodeloptions",
            name="vision_enabled",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 4.2.10 on 2024-05-26 12:35

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0042_serverchatsettings"),
    ]

    operations = [
        migrations.AlterField(
            model_name="chatmodeloptions",
            name="model_type",
            field=models.CharField(
                choices=[("openai", "Openai"), ("offline", "Offline"), ("anthropic", "Anthropic")],
                default="offline",
                max_length=200,
            ),
        ),
    ]



# Generated by Django 5.0.13 on 2025-03-23 04:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0086_alter_texttoimagemodelconfig_model_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="aimodelapi",
            name="api_key",
            field=models.CharField(max_length=4000),
        ),
    ]



# Generated by Django 5.0.7 on 2024-08-05 11:51

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0054_alter_agent_style_color"),
    ]

    operations = [
        migrations.AlterField(
            model_name="agent",
            name="style_icon",
            field=models.CharField(
                choices=[
                    ("Lightbulb", "Lighbulb"),
                    ("Health", "Health"),
                    ("Robot", "Robot"),
                    ("Aperture", "Aperture"),
                    ("GraduationCap", "Graduation Cap"),
                    ("Jeep", "Jeep"),
                    ("Island", "Island"),
                    ("MathOperations", "Math Operations"),
                    ("Asclepius", "Asclepius"),
                    ("Couch", "Couch"),
                    ("Code", "Code"),
                    ("Atom", "Atom"),
                    ("ClockCounterClockwise", "Clock Counter Clockwise"),
                    ("PencilLine", "Pencil Line"),
                    ("Chalkboard", "Chalkboard"),
                ],
                default="Lightbulb",
                max_length=200,
            ),
        ),
    ]



# Generated by Django 5.0.10 on 2025-01-15 11:05

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0079_searchmodelconfig_embeddings_inference_endpoint_type"),
    ]

    operations = [
        migrations.AddField(
            model_name="speechtotextmodeloptions",
            name="ai_model_api",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="database.aimodelapi",
            ),
        ),
    ]



import json
import logging
import os
from datetime import datetime, timezone

from asgiref.sync import sync_to_async
from fastapi import APIRouter, Request, Response
from starlette.authentication import requires

from khoj.database import adapters
from khoj.database.models import KhojUser, Subscription
from khoj.routers.helpers import update_telemetry_state
from khoj.utils import state

# Stripe integration for Khoj Cloud Subscription
if state.billing_enabled:
    import stripe

    stripe.api_key = os.getenv("STRIPE_API_KEY")
endpoint_secret = os.getenv("STRIPE_SIGNING_SECRET")
logger = logging.getLogger(__name__)
subscription_router = APIRouter()


@subscription_router.post("")
async def subscribe(request: Request):
    """Webhook for Stripe to send subscription events to Khoj Cloud"""
    event = None
    try:
        payload = await request.body()
        sig_header = request.headers["stripe-signature"]
        event = stripe.Webhook.construct_event(payload, sig_header, endpoint_secret)
    except ValueError as e:
        # Invalid payload
        raise e
    except stripe.error.SignatureVerificationError as e:
        # Invalid signature
        raise e

    event_type = event["type"]
    if event_type not in {
        "invoice.paid",
        "customer.subscription.updated",
        "customer.subscription.deleted",
    }:
        logger.warning(f"Unhandled Stripe event type: {event['type']}")
        return {"success": False}

    # Retrieve the customer's details
    subscription = event["data"]["object"]
    customer_id = subscription["customer"]
    customer = stripe.Customer.retrieve(customer_id)
    customer_email = customer["email"]
    user = None
    is_new = False

    # Handle valid stripe webhook events
    success = True
    if event_type in {"invoice.paid"}:
        # Mark the user as subscribed and update the next renewal date on payment
        subscription = stripe.Subscription.list(customer=customer_id).data[0]
        renewal_date = datetime.fromtimestamp(subscription["current_period_end"], tz=timezone.utc)
        user, is_new = await adapters.set_user_subscription(
            customer_email, is_recurring=True, renewal_date=renewal_date
        )
        success = user is not None
    elif event_type in {"customer.subscription.updated"}:
        user_subscription = await sync_to_async(adapters.get_user_subscription)(customer_email)

        renewal_date = None
        if subscription["current_period_end"]:
            renewal_date = datetime.fromtimestamp(subscription["current_period_end"], tz=timezone.utc)

        # Allow updating subscription status if paid user
        if user_subscription and user_subscription.renewal_date:
            # Mark user as unsubscribed or resubscribed
            is_recurring = not subscription["cancel_at_period_end"]
            user, is_new = await adapters.set_user_subscription(
                customer_email, is_recurring=is_recurring, renewal_date=renewal_date
            )
            success = user is not None
    elif event_type in {"customer.subscription.deleted"}:
        # Reset the user to trial state
        user, is_new = await adapters.set_user_subscription(
            customer_email, is_recurring=False, renewal_date=None, type=Subscription.Type.TRIAL
        )
        success = user is not None

    if user and is_new:
        update_telemetry_state(
            request=request,
            telemetry_type="api",
            api="create_user",
            metadata={"server_id": str(user.user.uuid)},
        )
        logger.log(logging.INFO, f"🥳 New User Created: {user.user.uuid}")

    logger.info(f'Stripe subscription {event["type"]} for {customer_email}')
    return {"success": success}


@subscription_router.patch("")
@requires(["authenticated"])
async def update_subscription(request: Request, operation: str):
    # Retrieve the customer's details
    email = request.user.object.email
    customers = stripe.Customer.list(email=email).auto_paging_iter()
    customer = next(customers, None)
    if customer is None:
        return {"success": False, "message": "Customer not found"}

    if operation == "cancel":
        customer_id = customer.id
        for subscription in stripe.Subscription.list(customer=customer_id):
            stripe.Subscription.modify(subscription.id, cancel_at_period_end=True)
        return {"success": True}

    elif operation == "resubscribe":
        subscriptions = stripe.Subscription.list(customer=customer.id).auto_paging_iter()
        # Find the subscription that is set to cancel at the end of the period
        for subscription in subscriptions:
            if subscription.cancel_at_period_end:
                # Update the subscription to not cancel at the end of the period
                stripe.Subscription.modify(subscription.id, cancel_at_period_end=False)
                return {"success": True}
        return {"success": False, "message": "No subscription found that is set to cancel"}

    return {"success": False, "message": "Invalid operation"}


@subscription_router.post("/trial", response_class=Response)
@requires(["authenticated"])
async def start_trial(request: Request) -> Response:
    user: KhojUser = request.user.object

    # Start a trial for the user
    updated_subscription = await adapters.astart_trial_subscription(user)

    # Return trial status as a JSON response
    return Response(
        content=json.dumps({"trial_enabled": updated_subscription is not None}),
        media_type="application/json",
        status_code=200,
    )



# Generated by Django 5.0.7 on 2024-08-16 18:09

from typing import List

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0056_chatmodeloptions_vision_enabled"),
        ("database", "0056_searchmodelconfig_cross_encoder_model_config"),
    ]
    operations: List[migrations.operations.base.Operation] = []



# Generated by Django 5.0.8 on 2024-09-19 15:59

import json
import pickle
import uuid

from django.db import migrations, models


def reverse_remove_bigint_id(apps, schema_editor):
    Conversation = apps.get_model("database", "Conversation")
    index = 1
    for conversation in Conversation.objects.all():
        conversation.id = index
        conversation.save()
        index += 1


def update_conversation_id_in_job_state(apps, schema_editor):
    try:
        DjangoJob = apps.get_model("django_apscheduler", "DjangoJob")
        Conversation = apps.get_model("database", "Conversation")

        for job in DjangoJob.objects.all():
            job_state = pickle.loads(job.job_state)
            kwargs = job_state.get("kwargs")
            conversation_id = kwargs.get("conversation_id") if kwargs else None
            automation_metadata = json.loads(job_state.get("name", "{}"))

            if not conversation_id:
                job.delete()

            if conversation_id:
                try:
                    conversation = Conversation.objects.get(id=conversation_id)
                    automation_metadata["conversation_id"] = str(conversation.temp_id)
                    name = json.dumps(automation_metadata)
                    job_state["name"] = name
                    job_state["kwargs"]["conversation_id"] = str(conversation.temp_id)
                    job.job_state = pickle.dumps(job_state)
                    job.save()
                except Conversation.DoesNotExist:
                    pass
    except LookupError as e:
        pass


def no_op(apps, schema_editor):
    pass


def disable_triggers(apps, schema_editor):
    schema_editor.execute('ALTER TABLE "database_conversation" DISABLE TRIGGER ALL;')


def enable_triggers(apps, schema_editor):
    schema_editor.execute('ALTER TABLE "database_conversation" ENABLE TRIGGER ALL;')


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0063_conversation_temp_id"),
    ]

    operations = [
        migrations.RunPython(no_op, reverse_code=enable_triggers),
        migrations.RunPython(update_conversation_id_in_job_state, reverse_code=no_op),
        migrations.RemoveField(
            model_name="conversation",
            name="id",
        ),
        migrations.RenameField(
            model_name="conversation",
            old_name="temp_id",
            new_name="id",
        ),
        migrations.AlterField(
            model_name="conversation",
            name="id",
            field=models.UUIDField(
                db_index=True, default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True
            ),
        ),
        migrations.RunPython(no_op, reverse_code=reverse_remove_bigint_id),
        migrations.RunPython(no_op, reverse_code=disable_triggers),
    ]
